#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/IdentifiedFinalState.hh"
#include "Rivet/Projections/PromptFinalState.hh"
#include "Rivet/Projections/DressedLeptons.hh"

namespace Rivet {

  class ATLAS_2016_I1468168 : public Analysis {
  public:

    DEFAULT_RIVET_ANA_CONSTRUCTOR(ATLAS_2016_I1468168)

    void init() {
      // Eta ranges
      Cut eta_full = (Cuts::abseta < 5.0) & (Cuts::pT >= 1.0*MeV);
      // Lepton cuts
      Cut lep_cuts = (Cuts::abseta < 2.5) & (Cuts::pT >= 25.0*GeV);

      // All final state particles
      FinalState fs(eta_full);

      // Get photons to dress leptons
      IdentifiedFinalState photons(fs);
      photons.acceptIdPair(PID::PHOTON);

      // Projection to find the electrons
      IdentifiedFinalState el_id(fs);
      el_id.acceptIdPair(PID::ELECTRON);
      PromptFinalState electrons(el_id);
      electrons.acceptTauDecays(true);
      addProjection(electrons, "electrons");
      DressedLeptons dressedelectrons(photons, electrons, 0.1, lep_cuts, true, true);
      addProjection(dressedelectrons, "dressedelectrons");

      // Projection to find the muons
      IdentifiedFinalState mu_id(fs);
      mu_id.acceptIdPair(PID::MUON);
      PromptFinalState muons(mu_id);
      muons.acceptTauDecays(true);
      addProjection(muons, "muons");
      DressedLeptons dressedmuons(photons, muons, 0.1, lep_cuts, true, true);
      addProjection(dressedmuons, "dressedmuons");

      _hist = bookHisto1D("passed_events", 1, 0, 1);
    }

    void analyze(const Event& event) {
      // Get the selected objects, using the projections.
      _dressedelectrons = applyProjection<DressedLeptons>(event, "dressedelectrons").dressedLeptons();
      _dressedmuons = applyProjection<DressedLeptons>(event, "dressedmuons").dressedLeptons();

      const double weight = event.weight();

      // Evaluate basic event selection
      bool pass_emu = (_dressedelectrons.size() == 1) && (_dressedmuons.size() == 1);

      // basic event selection requirements
      if (!pass_emu)  vetoEvent;

      // Fill histograms
      _hist->fill(0.5, weight);
    }

    void finalize() {
      // Normalize to cross-section
      const double sf(crossSection() / sumOfWeights());
      scale(_hist, sf);
    }

   private:
    /// @name Objects that are used by the event selection decisions
    vector<DressedLepton> _dressedelectrons, _dressedmuons;
    Histo1DPtr _hist;
  };

  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2016_I1468168);

}
